package cz.muni.fi.pb162.carrental;


/**
 * Trida predstavujici auto. Kazde auto je identifikovano
 * unikatnim ID typu String (napr. SPZ) a ma textovy retezec popisujici 
 * model auta.
 * 
 * Toto rozhrani rozsiruje rozhrani Comparable,
 * pricemz poradi aut je dano jejich identifikatory.
 * 
 * @author Radek Oslejsek 
 * @version 2007-11-16
 */
public interface Car extends Comparable<Car>
{
    /**
     * Metoda vraci identifikator auta (SPZ).
     * 
     * @return indetifikator auta (SPZ)
     */
    String getId();

    /**
     * Metoda vraci retezec popisujici model auta.
     * 
     * @return jmeno model auta
     */
    String getModel();
    
    /**
     * Zjisti, jestli je auto pujcene.
     * 
     * @return true poku je auto pujcene, jinak false
     */
    boolean isRented();
    
    /**
     * Pokud je auto volne, nastavi je na pujcene.
     * 
     * @return true pokud bylo auto volne, false pokud jiz bylo pujcene
     */
    boolean rentCar();
    
    /**
     * Pokud je auto pujcene, nastavi ho na volne.
     * 
     * @return true pokud bylo auto pujcene, false pokud bylo volne
     */
    boolean returnCar();
    
    /**
     * <p>Indikuje, zdali je nejaky objekt stejny jako toto auto.
     * Jiny objekt je stejny (ekvivalentni) jako toto auto prave
     * tehdy, pokud jde take o auto a ma stejny identifikator id.</p>
     * 
     * <p>Tato metoda musi samozrejme splnovat obecny kontrakt metody <a 
     * href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Object.html#equals(java.lang.Object)">Object.equals</a>.</p>
     *
     * @param obj Porovnavany objekt
     * @return true, jestli je tento objekt stejny jako argument obj
     */
    boolean equals(Object obj);
    
    /**
     * Tato metoda vraci hashovaci kod daneho objektu. Metoda musi splnovat obecny 
     * kontrakt metody <a 
     * href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Object.html#hashCode()">Object.hashCode()</a>.
     * 
     * @return hashovaci k daneho objektu
     */
    int hashCode();       
}
