package cz.muni.fi.pb162.carrental;

import java.util.Collection;
import java.util.SortedSet;
import java.util.List;
import java.util.Set;

/**
 * Autopujcovna.
 * 
 * @author Radek Oslejsek
 * @version 2007-11-16
 */

public interface CarRental
{
    /**
     * Metoda vytvori noveho zakaznika s danym jmenem. Zakaznik bude 
     * mit pridelen novy identifikator, ktery jeste zadny zakaznik 
     * v pujcovne nema.
     *
     * @param name jmeno noveho zakaznika
     * @return instanci tridy reprezentujici noveho zakaznika
     * @throws java.lang.IllegalArgumentException pokud ma paramentr name 
     * hodnotu null
     */
    Customer newCustomer(String name);
    
    /**
     * Vrati zakaznika s danym id.
     *
     * @param id identifikator zakaznika, ktereho ma metoda vratit.
     * @return instanci tridy reprezentujici zakaznikas danym identifikatorem 
     *         nebo null, pokud takovy zakaznika neexistuje.
     */
    Customer getCustomer(int id);
    
    /**
     * Vrati kolekci vsech zakazniku pujcovny. Kolekci neni mozne 
     * modifikovat. Tato metoda nikdy nevraci hodnotu null!
     *
     * @return kolekci vsech zakazniku autopujcovny
     */
    Collection<Customer> getCustomers();

    /**
     * Zaregistruje nove auto. Identifikator auta nesmi obsahovat mezeru.
     * 
     * @param id (SPZ) noveho auta
     * @param model model auta
     * @return nove vytvorene auto
     * @throws cz.muni.fi.pb162.carrental.CarRentalException Pokud auto s 
     *         danym ID jiz existuje
     * @throws java.lang.IllegalArgumentException Pokud parametr id obsahuje
     *         mezeru, nebo pokud ma parametr id nebo name honotu null
     */
    Car newCar(String id, String model) throws CarRentalException;
    
    /**
     * Vrati kolekci vsech aut pujcovny. Kolekci neni mozne modifikovat. 
     * Tato metoda nikdy nevraci hodnotu null! 
     *
     * @return kolekci vsech aut v autopujcove
     */
    Collection<Car> getCars();

    /**
     * Smaze vsechna auta, zakazniky i vypujcky.
     */
    void clear();

    /**
     * Vrati auta, ktera mel zakaznik pujcene. Auta budou setridena
     * podle identifikatoru a budou bez duplicit. Kolekci neni mozne
     * modifikovat. Tato metoda nikdy nevraci hodnot null!
     * 
     * @param customer zakaznik, jehoz auta nas zajimaji
     * @return auta nekdy pujcena zakaznikem
     * @throws cz.muni.fi.pb162.carrental.CarRentalException pokud zakaznik
     *         neexistuje
     * @throws java.lang.IllegalArgumentException pokud ma parametr customer
     *         hodnotu null
     */
    SortedSet<Car> getCustomerCars(Customer customer) throws CarRentalException;
    
    /**
     * Vrati seznam zakazniku, kteri meli nekdy pujcene dane auto. 
     * Seznam odpovida jednotlivym vypujckam vcetne poradi.
     * 
     * @param carId identifikator auta, jehoz vypujcky nas zajimaji
     * @return seznam zakazniku, kteri meli auto pujcene
     * @throws cz.muni.fi.pb162.carrental.CarRentalException pokud auto
     *         neexistuje
     * @throws java.lang.IllegalArgumentException pokud ma parametr car
     *         hodnotu null
     */
    List<Customer> getLoans(String carId) throws CarRentalException;

    /**
     * Pujci auto zakaznikovi, tj. nastavi auto na pujcene a zaznamena
     * vypujcku auta zakaznikem.
     * 
     * @param customer zakaznik, ktery si auto pujcuje
     * @param carId identifikator auta, ktere si chce zakaznik pujcit
     * @return false pokud je jiz auto pujcene, jinak true
     * @throws cz.muni.fi.pb162.carrental.CarRentalException pokud auto
     *         nebo zakaznik v pujcovne neexistuje
     * @throws java.lang.IllegalArgumentException pokud je nektery parametr null
     */
    boolean rentCar(Customer customer, String carId) throws CarRentalException;
    
    /**
     * Vrati auto, tj. nastavi jeho stav na "volne".
     * 
     * @param carId identifikator auta, ktere zakaznik vraci
     * @return true pokud auto bylo pujceno, jinak false
     * @throws cz.muni.fi.pb162.carrental.CarRentalException pokud auto
     *         v pujcovne neexistuje
     * @throws java.lang.IllegalArgumentException pokud je parametr null
     */
    boolean returnCar(String carId) throws CarRentalException ;
}
