package cz.muni.fi.pb162.carrental;

import java.util.*;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

public class CarRentalTest extends TestCase {
    
    public CarRentalTest(String testName) {
        super(testName);
    }

    public static Test suite() {
        TestSuite suite = new TestSuite(CarRentalTest.class);
        return suite;
    }
    
    protected void assertException(String when, Throwable t) {
        assertTrue("Vyjimka " + t.getClass().getName() + " vyvolana pri " +
                "pokusu o " + when + " neobsahuje podrobny popis!",
                t.getMessage() != null &&
                t.getMessage().trim().length() > 0);
    }

    public void testRules() {
        BasicRulesTester.testMethodsAndAttributes(CarImpl.class);
        BasicRulesTester.testMethodsAndAttributes(CustomerImpl.class);
        BasicRulesTester.testMethodsAndAttributes(CarRentalImpl.class);
    }
    
    public final void testNewCar() throws CarRentalException{
        CarRental cr = new CarRentalImpl();
        Car car = cr.newCar("id","model");

        assertNotNull("Metoda newCar() vraci null", car);
        assertEquals("Nove vytvorene auto ma jine id, nez bylo zadano", "id",
            car.getId());
        assertEquals("Nove vytvorene auto ma nastavene jiny model, nez bylo zadano","model",
            car.getModel());        
        assertFalse("Nove auto nema byt ve stavu \"pujcene\"", car.isRented());
        
        assertTrue("Kolekce aut vracena metodou getCars() neobsahuje" +
        " nove vytvorene auto",cr.getCars().contains(car));
        
        try {
            cr.newCar("id", "aaa");
            fail("Pri pokusu o vytvoreni auta s jiz existjici id se nevyhodila vyjimka");
        } catch (CarRentalException ex) {
            // OK
            assertException("vytvoreni auta s jiz existjicim id", ex);
        } catch (Exception ex) {
            fail("Pokus o pridani auta s jiz existujicim id vyvol " +
                 "misto CarRentalException vyjimku " + 
                 ex.getClass().getName());
        }

        try {
            cr.newCar("id", null);
            fail("Pri pokusu o vytvoreni auta s praznym parametrem model se nevyhodila vyjimka");
        } catch (IllegalArgumentException ex) {
            assertException("vytvoreni auta s parametrem model nastavenym na null", ex);
        } catch (NullPointerException ex) {
            assertException("vytvoreni auta s parametrem model nastavenym na null", ex);
        } catch (Exception ex) {
            fail("Pokus o pridani auta s praznym parametrem model vyvolal " +
                 "nespravnou vyjimku " + ex.getClass().getName());
        }
        
        try {
            cr.newCar("id bla", "aaa");
            fail("Pri pokusu o vytvoreni auta s mezerou v id se nevyhodila vyjimka");
        } catch (IllegalArgumentException ex) {
            assertException("vytvoreni auta s mezerou v id", ex);
        } catch (NullPointerException ex) {
            assertException("vytvoreni auta s mezerou v id", ex);
        } catch (Exception ex) {
            fail("Pokus o pridani auta s mezerou v id vyvolal " +
                 "nespravnou vyjimku " + ex.getClass().getName());
        }

        try {
            new CarRentalImpl().newCar(null, "model");
            fail("Pri pokusu o vytvoreni auta s praznym ID se nevyhodila vyjimka");
        } catch (IllegalArgumentException ex) {
            assertException("vytvoreni auta s parametrem id nastavenym na null", ex);
        } catch (NullPointerException ex) {
            assertException("vytvoreni auta s parametrem id nastavenym na null", ex);
        } catch (Exception ex) {
            fail("Pokus o pridani auta s prazdnym id vyvolal " +
                 "nespravnou vyjimku " + ex.getClass().getName());
        }
    }

    public final void testCarEquals() throws CarRentalException {
        CarRental cr = new CarRentalImpl();
        Car car1 = cr.newCar("id1", "model1");
        Car car2 = new CarRentalImpl().newCar("id1","model2");
        Car car3 = cr.newCar("id2", "model1");

        assertTrue("Metoda equals vraci false, i kdyz je jejim parametrem " +
            "instance auta se stejnym id (chyba v metode CarImpl.equals)",
            car1.equals(car2));
        assertFalse("Metoda equals vraci true, i kdyz je jejim parametrem " +
            "instance auta s jinym id (chyba v metode CarImpl.equals)",
            car1.equals(car3));
        
        try {
            assertFalse("Metoda equals vraci true, i kdyz je jejim parametrem " +
            "hodnota null (chyba v metode CarImpl.equals)",
            car1.equals(null));
        } catch (Exception ex) {
            fail("Kdyz se zavola metoda CarImpl.equals() s parametrem null," 
                    + " metoda nevrati false, ale vyhodi vyjimku " + 
                    ex.getClass().getName());
        }
        
        try {
            assertFalse("Metoda equals vraci true, i kdyz je jejim parametrem " +
            "objekt jineho typu (chyba v metode CarImpl.equals)",
            car1.equals(new Object()));
        } catch (Exception ex) {
            fail("Kdyz se zavola metoda CarImpl.equals() s parametrem," 
                    + " jineho typu, metoda nevrati false, ale vyhodi vyjimku " + 
                    ex.getClass().getName());
        }        
    }
    
    public final void testCarHashCode() throws CarRentalException {
        CarRental cr = new CarRentalImpl();
        Car car1 = cr.newCar("id1", "model1");
        Car car2 = new CarRentalImpl().newCar("id1","model2");
        Car car3 = cr.newCar("id2", "model1");

        assertFalse("Metoda CarImpl.hashCode() neni prekryta, takze " +
            "porusuje obecny kontrakt metody Object.hashCode()",
            car1.hashCode() == System.identityHashCode(car1));
        assertEquals("Metoda CarImpl.hashCode() vraci pro dve stejna auta" +
            " ruznou hodnotu, takze porusuje obecny kontrakt metody " +
            "Object.hashCode() ",
            car1.hashCode(), car2.hashCode());
        assertFalse("Metoda CarImpl.hashCode() vraci pro dve ruzna auta " +
            "stejny kod; to sice neporusuje obecny kontrakt metody " +
            "Object.hashCode(), ale v tomto pripade lze snadno vytvorit " +
            "temer bezkolizni implementaci teto metody",
            car1.hashCode() == car3.hashCode());                
    }

    public final void testCarRentCarReturn() throws CarRentalException {
        Car car = new CarRentalImpl().newCar("id","model");
        assertTrue("CarImpl.rentCar() vraci false u noveho auta", car.rentCar());
        assertTrue("CarImpl.isRented() vraci false u pujceneho auta", car.isRented());
        assertFalse("CarImpl.rentCar() vraci true u pujceneho auta", car.rentCar());
        assertTrue("CarImpl.isRented() vraci false po druhem pokusu o pojceni auta", car.isRented());
        assertTrue("CarImpl.returnCar() vraci false pri vraceni pujceneho auta",car.returnCar());
        assertFalse("CarImpl.isRented() vraci true u vraceneho auta", car.isRented());
        assertFalse("CarImpl.returnCar() vraci true pri vraceni jiz vraceneho auta",car.returnCar());
        assertFalse("CarImpl.isRented() vraci true u dvakrat vraceneho auta", car.isRented());
    }
    
    private static void assertSameSign(String msg, int n1, int n2) {
        assertTrue(msg,
                // musi byt osetrena i nula, proto zdanlive 
                // zbytecna druha cast nasledujiciho vyrazu
                ((n1 < 0) == (n2 < 0)) && ((n1 > 0) == (n2 > 0)));
    }

    public final void testCarCompareTo() throws CarRentalException {
        CarRental cr = new CarRentalImpl();
        Car car1 = cr.newCar("id1", "model1");
        Car car2 = new CarRentalImpl().newCar("id1","model2");
        Car car3 = cr.newCar("id2", "model1");
        
        String msg = "Metoda CarImpl.compareTo() nezajisti usporadani podle id";
        assertSameSign(msg, car1.getId().compareTo(car3.getId()), car1.compareTo(car3));
        assertSameSign(msg, car3.getId().compareTo(car1.getId()), car3.compareTo(car1));
        assertEquals("Metoda CarImp.compareTo() se nechova spravne, kdyz " +
            "porovnava dva stejne objekty",0,car1.compareTo(car1));
        assertEquals("Metoda CarImp.compareTo() se nechova spravne, kdyz " +
            "porovnava dva stejne objekty",0,car1.compareTo(car2));
    }
    
    public final void testCustomer() {
        CarRental cr = new CarRentalImpl();
        Customer c1 = cr.newCustomer("name name");

        assertNotNull("Metoda newCustomer() vraci null", c1);
        assertEquals("Nove vytvoreny zakaznik ma jine jmeno, nez bylo zadano","name name",c1.getName());
        
        assertNotNull("Metoda getCustomer s identifikatorem nove vytvoreneho " +
            "zakaznika vraci null", cr.getCustomer(c1.getId()));
            
        assertEquals("Metoda getCustomer vraci zakaznika s jinym id",
            cr.getCustomer(c1.getId()).getId(), c1.getId());
        
        assertTrue("Kolekce zakazniku vracena metodou getCustomers() neobsahuje" +
            " nove vytvoreneho zakaznika",cr.getCustomers().contains(c1));

        Customer c2 = new CarRentalImpl().newCustomer("name2");
        assertEquals("Metoda newCustomer() se chova " +
            "nedeterministicky; prvni vytvoreny zakaznik ma pokazde jine id",
            c1.getId(),c2.getId());

        Customer c3 = cr.newCustomer("name name");
        assertFalse("Metoda CarRentalImpl.newCustomer prirazuje stejna ID",
            c1.getId() == c3.getId());
            
        try {
            cr.newCustomer(null);
            fail("Pri pokusu o vytvoreni zakaznika s praznym jmenem se nevyhodila vyjimka");
        } catch (IllegalArgumentException ex) {
            assertException("vytvoreni zakaznika s parametrem name nastavenym na null", ex);
        } catch (NullPointerException ex) {
            assertException("vytvoreni zakaznika s parametrem name nastavenym na null", ex);
        } catch (Exception ex) {
            fail("Pokus o pridani zakaznika s praznym parametrem name vyvolal " +
                 "nespravnou vyjimku " + ex.getClass().getName());
        }
        
        assertTrue("Metoda equals vraci false, i kdyz je jejim parametrem " +
            "instance zakaznika se stejnym id (chyba v metode CustomerImpl.equals)",
            c1.equals(c2));
        assertFalse("Metoda equals vraci true, i kdyz je jejim parametrem " +
            "instance zakaznika s jinym id (chyba v metode CustomerImpl.equals)",
            c1.equals(c3));
        
        try {
            assertFalse("Metoda equals vraci true, i kdyz je jejim parametrem " +
            "hodnota null (chyba v metode CustomerImpl.equals)",
            c1.equals(null));
        } catch (Exception ex) {
            fail("Kdyz se zavola metoda CustomerImpl.equals() s parametrem null," 
                    + " metoda nevrati false, ale vyhodi vyjimku " + 
                    ex.getClass().getName());
        }
        
        try {
            assertFalse("Metoda equals vraci true, i kdyz je jejim parametrem " +
            "objekt jineho typu (chyba v metode CustomerImpl.equals)",
            c1.equals(new Object()));
        } catch (Exception ex) {
            fail("Kdyz se zavola metoda CustomerImpl.equals() s parametrem," 
                    + " jineho typu, metoda nevrati false, ale vyhodi vyjimku " + 
                    ex.getClass().getName());
        }        
        
        assertFalse("Metoda CustomerImpl.hashCode() neni prekryta, takze " +
            "porusuje obecny kontrakt metody Object.hashCode()",
            c1.hashCode() == System.identityHashCode(c1));
        assertEquals("Metoda CustomerImpl.hashCode() vraci pro dve stejna auta" +
            " ruznou hodnotu, takze porusuje obecny kontrakt metody " +
            "Object.hashCode() ",
            c1.hashCode(), c2.hashCode());
        assertFalse("Metoda CustomerImpl.hashCode() vraci pro dve ruzna auta " +
            "stejny kod; to sice neporusuje obecny kontrakt metody " +
            "Object.hashCode(), ale v tomto pripade lze snadno vytvorit " +
            "bezkolizni implementaci teto metody",
            c1.hashCode() == c3.hashCode());                        
    }    
    
    public final void testNewCarRental() throws CarRentalException {
        CarRental cr1 = new CarRentalImpl();
        CarRental cr2 = new CarRentalImpl();
        assertNotNull("Metoda CarRentalImpl.getCustomers() vraci null, ackoliv null nikdy vracet nema",
            cr1.getCustomers());
        assertNotNull("Metoda CarRentalImpl.getCars() vraci null, ackoliv null nikdy vracet nema",
            cr1.getCars());
        assertTrue("Metoda CarRentalImpl.getCustomers() vraci neprazdnou kolekci, " +
            "ackoliv jeste nebyl vytvoren zadny zakaznik",cr1.getCustomers().isEmpty());
        assertTrue("Metoda CarRentalImpl.getCars() vraci neprazdnou kolekci, " +
            "ackoliv jeste nebylo vytvoreno zadne auto",cr1.getCars().isEmpty());
        cr2.newCustomer("Test customer");
        cr2.newCar("carid","Test model");
        assertTrue("Metoda CarRentalImpl.getCustomers() vraci neprazdnou kolekci, " +
            "ackoliv jeste nebyl vytvoren zadny zakaznik (pravdepodobne mate kolekci " +
            "zakazniku realizovanou jako staticky atribut)",cr1.getCustomers().isEmpty());   
        assertTrue("Metoda CarRentalImpl.getCars() vraci neprazdnou kolekci, " +
            "ackoliv jeste nebylo vytvoreno zadne auto (pravdepodobne mate kolekci " +
            "aut realizovanou jako staticky atribut)",cr1.getCars().isEmpty());    
    }
    
    public void testGetCarsOfNonExistentCustomer() {
        try {
            new CarRentalImpl().getCustomerCars(null);
            fail("Volani metody getCustomerCars s parametrem null " +
                    "nezpusobi zadnou vyjimku");
        } catch (NullPointerException ex) {
            // All is ok
            assertException("volani metody getCustomerCars s parametrem null",ex);
        } catch (IllegalArgumentException ex) {
            // All is ok
            assertException("volani metody getCustomerCars s parametrem null",ex);
        } catch (Exception ex) {
            fail("Volani metody getCustomerCars s parametrem null vyvola " +
                    "misto IllegalArgumentException vyjimku " + 
                    ex.getClass().getName());
        }
        
        try {
            new CarRentalImpl().getCustomerCars(new TestCustomer(234));
            fail("Volani metody getCustomerCars pro neexistujiciho zakaznika " +
                    "nezpusobi zadnou vyjimku");
        } catch (CarRentalException ex) {
            // All is ok
            assertException("volani metody getCustomerCars pro neexistujiciho zakaznika",ex);
        } catch (Exception ex) {
            fail("Volani metody getCustomerCars pro neexistujiciho zakaznika vyvola " +
                    "misto CarRentalException vyjimku " + ex.getClass().getName());
        }
    }
    
    public void testGetLoansOfNonExistentCar() {
        try {
            new CarRentalImpl().getLoans(null);
            fail("Volani metody getLoans s parametrem null " +
                    "nezpusobi zadnou vyjimku");
        } catch (NullPointerException ex) {
            // All is ok
            assertException("volani metody getLoans s parametrem null",ex);
        } catch (IllegalArgumentException ex) {
            // All is ok
            assertException("volani metody getLoans s parametrem null",ex);
        } catch (Exception ex) {
            fail("Volani metody getLoans s parametrem null vyvola " +
                    "misto IllegalArgumentException vyjimku " + 
                    ex.getClass().getName());
        }
        
        try {
            new CarRentalImpl().getLoans("234");
            fail("Volani metody getLoans pro neexistujici auto " +
                    "nezpusobi zadnou vyjimku");
        } catch (CarRentalException ex) {
            // All is ok
            assertException("volani metody getLoans pro neexistujici auto",ex);
        } catch (Exception ex) {
            fail("Volani metody getLoans pro neexistujici auto vyvola " +
                    "misto CarRentalException vyjimku " + ex.getClass().getName());
        }
    }
    
    public void testCarRental() throws CarRentalException {
        
        CarRental cr = new CarRentalImpl();
        Car car1 = cr.newCar("AAA1", "Porshe");
        Customer cust1 = cr.newCustomer("Pepa z depa");
        Customer cust2 = cr.newCustomer("Tomas Jedno");
        
        Car car2 = cr.newCar("BBB2", "Audi");        
        Car car3 = cr.newCar("CCC3", "Skoda");
                
        assertFalse("Nove auto je chybne nastaveno na pujcene", car1.isRented());
        assertFalse("Nove auto je chybne nastaveno na pujcene", car2.isRented());
        assertFalse("Nove auto je chybne nastaveno na pujcene", car3.isRented());

        Customer strangeCustomer = new TestCustomer(cust1.getId() + 1222);
        Car strangeCar = new TestCar("XXX0");

        SortedSet<Car> cust1cars = new TreeSet<Car>();
        SortedSet<Car> cust2cars = new TreeSet<Car>();
        List<Customer> car1custs = new ArrayList<Customer>();
        List<Customer> car2custs = new ArrayList<Customer>();
        List<Customer> car3custs = new ArrayList<Customer>();

        try {
            assertEquals("Seznam aut pujcenych zakaznikem "+ cust1 +" neodpovida",
                cust1cars, cr.getCustomerCars(cust1));
        } catch (Exception ex) {
            fail("CarRentalImpl.getCustomerCars() vyhodila vyjimku " + ex.getClass().getName() + " pro existujiciho zakaznika");
        }
        assertEquals("Seznam aut pujcenych zakaznikem "+ cust2 +" neodpovida",
                cust2cars, cr.getCustomerCars(cust2));
        assertEquals("Seznam zakazniku, kteri si pujcili auto "+ car1 +" neodpovida",
                car1custs, cr.getLoans(car1.getId()));
        assertEquals("Seznam zakazniku, kteri si pujcili auto "+ car2 +" neodpovida",
                car2custs, cr.getLoans(car2.getId()));
        assertEquals("Seznam zakazniku, kteri si pujcili auto "+ car3 +" neodpovida",
                car3custs, cr.getLoans(car3.getId()));

        try {
            assertTrue("CarRentalImpl.rentCar() vraci false pri pokusu o pujceni volneho auta", cr.rentCar(cust1, car1.getId()));
            assertTrue("CarRentalImpl.rentCar() vraci false pri pokusu o pujceni volneho auta", cr.rentCar(cust1, car2.getId()));
            assertTrue("CarRentalImpl.rentCar() nenastavuje auto na pujcene", car1.isRented());
            assertTrue("CarRentalImpl.rentCar() nenastavuje auto na pujcene", car2.isRented());
        } catch (Exception ex) {
            fail("Neocekavana vyjimka pri volani metody CarRentalImpl.rentCar(): " + ex.getClass().getName());
        }
        try {
            assertTrue("CarRentalImpl.returnCar() vraci false pri vraceni pujceneho auta", cr.returnCar(car1.getId()));
            assertTrue("CarRentalImpl.returnCar() vraci false pri vraceni pujceneho auta", cr.returnCar(car2.getId()));
            assertFalse("CarRentalImpl.returnCar() nenastavuje auto na pujcene", car1.isRented());
            assertFalse("CarRentalImpl.returnCar() nenastavuje auto na pujcene", car2.isRented());
        } catch (Exception ex) {
            fail("Neocekavana vyjimka pri volani metody CarRentalImpl.returnCar()" + ex.getClass().getName());
        }
        cust1cars.add(car1);
        cust1cars.add(car2);
        car1custs.add(cust1);
        car2custs.add(cust1);

        assertEquals("Seznam aut pujcenych zakaznikem "+ cust1 +" neodpovida",
                cust1cars, cr.getCustomerCars(cust1));
        assertEquals("Seznam aut pujcenych zakaznikem "+ cust2 +" neodpovida",
                cust2cars, cr.getCustomerCars(cust2));
        assertEquals("Seznam zakazniku, kteri si pujcili auto "+ car1 +" neodpovida",
                car1custs, cr.getLoans(car1.getId()));
        assertEquals("Seznam zakazniku, kteri si pujcili auto "+ car2 +" neodpovida",
                car2custs, cr.getLoans(car2.getId()));
        assertEquals("Seznam zakazniku, kteri si pujcili auto "+ car3 +" neodpovida",
                car3custs, cr.getLoans(car3.getId()));

        assertTrue(cr.rentCar(cust1, car3.getId()));
        assertTrue(cr.returnCar(car3.getId()));
        cust1cars.add(car3);
        car3custs.add(cust1);

        assertEquals("Seznam aut pujcenych zakaznikem "+ cust1 +" neodpovida",
                cust1cars, cr.getCustomerCars(cust1));
        assertEquals("Seznam aut pujcenych zakaznikem "+ cust2 +" neodpovida",
                cust2cars, cr.getCustomerCars(cust2));
        assertEquals("Seznam zakazniku, kteri si pujcili auto "+ car1 +" neodpovida",
                car1custs, cr.getLoans(car1.getId()));
        assertEquals("Seznam zakazniku, kteri si pujcili auto "+ car2 +" neodpovida",
                car2custs, cr.getLoans(car2.getId()));
        assertEquals("Seznam zakazniku, kteri si pujcili auto "+ car3 +" neodpovida",
                car3custs, cr.getLoans(car3.getId()));

        assertTrue(cr.rentCar(cust2, car3.getId()));
        assertTrue(cr.returnCar(car3.getId()));
        cust2cars.add(car3);
        car3custs.add(cust2);

        assertEquals("Seznam aut pujcenych zakaznikem "+ cust1 +" neodpovida",
                cust1cars, cr.getCustomerCars(cust1));
        assertEquals("Seznam aut pujcenych zakaznikem "+ cust2 +" neodpovida",
                cust2cars, cr.getCustomerCars(cust2));
        assertEquals("Seznam zakazniku, kteri si pujcili auto "+ car1 +" neodpovida",
                car1custs, cr.getLoans(car1.getId()));
        assertEquals("Seznam zakazniku, kteri si pujcili auto "+ car2 +" neodpovida",
                car2custs, cr.getLoans(car2.getId()));
        assertEquals("Seznam zakazniku, kteri si pujcili auto "+ car3 +" neodpovida",
                car3custs, cr.getLoans(car3.getId()));

        assertTrue(cr.rentCar(cust1, car1.getId()));
        assertTrue(cr.rentCar(cust2, car2.getId()));
        cust1cars.add(car1);
        cust2cars.add(car2);
        car1custs.add(cust1);
        car2custs.add(cust2);
                
        assertEquals("Seznam aut pujcenych zakaznikem "+ cust1 +" neodpovida",
                cust1cars, cr.getCustomerCars(cust1));
        assertEquals("Seznam aut pujcenych zakaznikem "+ cust2 +" neodpovida",
                cust2cars, cr.getCustomerCars(cust2));
        assertEquals("Seznam zakazniku, kteri si pujcili auto "+ car1 +" neodpovida",
                car1custs, cr.getLoans(car1.getId()));
        assertEquals("Seznam zakazniku, kteri si pujcili auto "+ car2 +" neodpovida",
                car2custs, cr.getLoans(car2.getId()));
        assertEquals("Seznam zakazniku, kteri si pujcili auto "+ car3 +" neodpovida",
                car3custs, cr.getLoans(car3.getId()));

        try {
            assertFalse("CarRentalImpl.rentCar() vraci true pri pujcovani jiz pujceneho auta", 
                cr.rentCar(cust1, car2.getId()));
            assertEquals("Seznam aut pujcenych zakaznikem "+ cust1 +" neodpovida",
                cust1cars, cr.getCustomerCars(cust1));
            assertEquals("Seznam zakazniku, kteri si pujcili auto "+ car2 +" neodpovida",
                car2custs, cr.getLoans(car2.getId()));
            assertTrue("CarRentalImpl.rentCar() nastavi auto na volne pri pokusu o nasebne pujceni", car2.isRented());
        } catch (Exception ex) {
            fail("Neocekavana vyjimka pri pujcovani jiz pujceneho auta metodou CarRentalImpl.rentCar(): " + 
                ex.getClass().getName());
        }

        try {
            cr.rentCar(cust1, strangeCar.getId());
            fail("Je mozne pujcit zakaznikovi cizi (neexistujici) auto");
        } catch (CarRentalException ex) {
            // All is ok
            assertException("pujceni neexistujiciho auta",ex);
        } catch (Exception ex) {
            fail("Pokus o pujceni ciziho (neexistujiciho) auta vyvola " +
                    "misto CarRentalException vyjimku " + ex.getClass().getName());
        }
        
        try {
            assertFalse(car3.isRented()); // pro jistotu
            cr.rentCar(strangeCustomer, car3.getId());
            fail("Je mozne pujcit auto cizimu (neexistujicimu) zakaznikovi");
        } catch (CarRentalException ex) {
            // All is ok
            assertFalse("CarRentalImpl.rentCar() meni auto na pujcene, prestoze zakaznik neexistuje", car3.isRented());
            assertException("pujceni auta cizimu (neexistujicimu) zakaznikovi",ex);
        } catch (Exception ex) {
            fail("Pokus o pujceni auta cizimu (neexistujicimu) zakaznikovi vyvola " +
                    "misto CarRentalException vyjimku " + ex.getClass().getName());
        }

        try {
            cr.rentCar(cust1, (String)null);
            fail("Je mozne pujcit zakaznikovi prazdne (null) auto");
        } catch (NullPointerException ex) {
            // All is ok
            assertException("pujceni prazdneho (null) auta",ex);
        } catch (IllegalArgumentException ex) {
            // All is ok
            assertException("pujceni prazdneho (null) auta",ex);
        } catch (Exception ex) {
            fail("Pokus o pujceni prazdneho (null) auta vyvola " +
                    "misto IllegalArgumentException vyjimku " + ex.getClass().getName());
        }

        try {
            cr.rentCar(null, car3.getId());
            fail("Je mozne pujcit auto prazdnemu (null) zakaznikovi");
        } catch (NullPointerException ex) {
            // All is ok
            assertFalse("CarRentalImpl.rentCar() meni auto na pujcene, prestoze zakaznik je null", car3.isRented());
            assertException("pujceni auta prazdnemu (null) zakaznikovi",ex);
        } catch (IllegalArgumentException ex) {
            // All is ok
            assertFalse("CarRentalImpl.rentCar() meni auto na pujcene, prestoze zakaznik je null", car3.isRented());
            assertException("pujceni auta prazdnemu (null) zakaznikovi",ex);
        } catch (Exception ex) {
            fail("Pokus o pujceni auta prazdnemu (null) zakaznikovi vyvola " +
                    "misto IllegalArgumentException vyjimku " + ex.getClass().getName());
        }
        
        try {
            assertTrue("CarRentalImpl.returnCar() vraci chybne false", cr.returnCar(car1.getId()));
        } catch (Exception ex) {
            fail("Neocekavana vyjimka pri volani metody CarRentalImpl.returnCar()" + ex.getClass().getName());
        }
        try {
            assertFalse("CarRentalImpl.returnCar() vraci true pri vraceni jiz vraceneho auta", cr.returnCar(car1.getId()));
            assertFalse("CarRentalImpl.returnCar() nenastavuje spravne stav vraceneho auta", car1.isRented());
        } catch (Exception ex) {
            fail("CarRentalImpl.returnCar() vyhodi neocekavanou vyjimku pri vraceni jiz vraceneho auta: " + ex.getClass().getName());
        }
        try {
            cr.returnCar(strangeCar.getId());
            fail("CarRentalImpl.returnCar() nevyhodi pozadovanou vyjimku pri vraceni ciziho (neexistujici) auta");
        } catch (CarRentalException ex) {
            // All is ok
            assertException("vraceni neexistujiciho auta",ex);
        } catch (Exception ex) {
            fail("Pokus o vraceni ciziho (neexistujiciho) auta vyvola " +
                    "misto CarRentalException vyjimku " + ex.getClass().getName());
        }
        try {
            cr.returnCar((String)null);
            fail("CarRentalImpl.returnCar() nevyhodi pozadovanou vyjimku pri vraceni prazdneho (null) auta");
        } catch (NullPointerException ex) {
            // All is ok
            assertException("vraceni prazdneho (null) auta",ex);
        } catch (IllegalArgumentException ex) {
            // All is ok
            assertException("vraceni prazdneho (null) auta",ex);
        } catch (Exception ex) {
            fail("Pokus o vraceni prazdneho (null) auta vyvola " +
                    "misto IllegalArgumentException vyjimku " + ex.getClass().getName());
        }
        
        assertEquals("Seznam aut pujcenych zakaznikem "+ cust1 +" neodpovida",
                cust1cars, cr.getCustomerCars(cust1));
        assertEquals("Seznam aut pujcenych zakaznikem "+ cust2 +" neodpovida",
                cust2cars, cr.getCustomerCars(cust2));
        assertEquals("Seznam zakazniku, kteri si pujcili auto "+ car1 +" neodpovida",
                car1custs, cr.getLoans(car1.getId()));
        assertEquals("Seznam zakazniku, kteri si pujcili auto "+ car2 +" neodpovida",
                car2custs, cr.getLoans(car2.getId()));
        assertEquals("Seznam zakazniku, kteri si pujcili auto "+ car3 +" neodpovida",
                car3custs, cr.getLoans(car3.getId()));
    }
    
    public final void testClear() throws CarRentalException {
        CarRental cr = new CarRentalImpl();
        Customer cust = cr.newCustomer("test customer");
        assertFalse("Po vytvoreni zakaznika je vraceny seznam zakazniku prazdny", 
            cr.getCustomers().isEmpty());
        Car car = cr.newCar("carid", "car model");
        assertFalse("Po vytvoreni auta je vraceny seznam aut prazdny", 
            cr.getCars().isEmpty());
        assertTrue(cr.rentCar(cust, car.getId()));
        
        cr.clear();
        assertTrue("Po volani metody clear neni seznam zakazniku prazdny", 
            cr.getCustomers().isEmpty());
        assertTrue("Po volani metody clear neni seznam aut prazdny", 
            cr.getCars().isEmpty());

        try {
            cr.getCustomerCars(cust);
            fail("Volani metody clear nevymaze seznam aut pujcenych zakaznikem");
        } catch(CarRentalException ex) {
            // All is ok
        }

        try {
            cr.getLoans(car.getId());
            fail("Volani metody clear nevymaze vypujcky auta");
        } catch(CarRentalException ex) {
            // All is ok
        }
    }    
    
    public void testReadOnlyCollections() throws CarRentalException {
        
        CarRental cr = new CarRentalImpl();
        try {
            cr.getCustomers().clear();
            fail("Metoda getCustomers() nevraci read-only kolekci!");
        } catch(UnsupportedOperationException ex) {
            // OK
        }
        try {
            cr.getCars().clear();
            fail("Metoda getCars() nevraci read-only kolekci!");
        } catch(UnsupportedOperationException ex) {
            // OK
        }
        
        Customer cust = cr.newCustomer("aaa");
        Car car = cr.newCar("aaa", "bbb");

        try {
            cr.getCustomerCars(cust).clear();
            fail("Metoda getCustomerCars() nevraci read-only kolekci!");
        } catch(UnsupportedOperationException ex) {
            // OK
        }

        try {
            cr.getLoans(car.getId()).clear();
            fail("Metoda getLoans() nevraci read-only kolekci!");
        } catch(UnsupportedOperationException ex) {
            // OK
        }
    }
    
    
    
    
    protected static class TestCustomer implements Customer {

        private int id;
        
        protected TestCustomer(int id) {
            this.id = id;
        }
        
        public String getName() {
            return "Zakaznik cislo " + id;
        }

        public int getId() {
            return id;
        }
    }

    private class TestCar implements Car {

        String id;
        boolean rented;

        TestCar(String id) {
            this.id = id;
            rented = false;
        }

        public String getModel() {
            return "Test car";
        }

        public String getId() {
            return id;
        }
        
        public boolean isRented() {
            return rented;
        }

        public boolean rentCar() {
            return (rented) ? false : (rented = true);
        }
    
        public boolean returnCar() {
            return (rented) ? !(rented = false) : false;
        }
                
        public int compareTo(Car car) {
            return id.compareTo(car.getId());
        }
    }

}