package cz.muni.fi.pb162.carrental;


/**
 * Trida predstavujici zakanika pujcovny aut. Kazdy zakaznik je identifikovany
 * unikatnim ID typu int a ma jmeno.
 * 
 * @author Radek Oslejsek
 * @version 2007-11-16
 */

public interface Customer
{
    /**
     * Metoda vraci identifikator zakaznika.
     * 
     * @return indetifikator zakaznika
     */
    int getId();

    /**
     * Metoda vraci jmeno zakaznika.
     * 
     * @return jmeno zakaznika
     */
    String getName();
    
    /**
     * <p>Indikuje, zdali je nejaky objekt stejny jako tento zakaznik.
     * Jiny objekt je stejny (ekvivalentni) jako tento zakaznik prave
     * tehdy, pokud jde take o zakaznika a ten ma stejny identifikator id.</p>
     * 
     * <p>Tato metoda musi samozrejme splnovat obecny kontrakt metody <a 
     * href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Object.html#equals(java.lang.Object)">Object.equals</a>.</p>
     *
     * @param obj Porovnavany objekt
     * @return true, jestli je tento objekt stejny jako argument obj
     */
    boolean equals(Object obj);
    
    /**
     * Tato metoda vraci hashovaci kod daneho objektu. Metoda musi splnovat obecny 
     * kontrakt metody <a 
     * href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Object.html#hashCode()">Object.hashCode()</a>.
     * 
     * @return hashovaci k daneho objektu
     */
    int hashCode();       
}
